#!/usr/bin/python2.4
#############################################################################
##
## Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
## Copyright (C) 2008 Benoit Sigoure
##
## This file is based on the demonstration applications of the Qt Toolkit.
##
## This file may be used under the terms of the GNU General Public
## License versions 2.0 or 3.0 as published by the Free Software
## Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
## included in the packaging of this file.  Alternatively you may (at
## your option) use any later version of the GNU General Public
## License if such license has been publicly approved by Trolltech ASA
## (or its successors, if any) and the KDE Free Qt Foundation. In
## addition, as a special exception, Trolltech gives you certain
## additional rights. These rights are described in the Trolltech GPL
## Exception version 1.1, which can be found at
## http://www.trolltech.com/products/qt/gplexception/ and in the file
## GPL_EXCEPTION.txt in this package.
##
## Please review the following information to ensure GNU General
## Public Licensing requirements will be met:
## http://trolltech.com/products/qt/licenses/licensing/opensource/. If
## you are unsure which license is appropriate for your use, please
## review the following information:
## http://trolltech.com/products/qt/licenses/licensing/licensingoverview
## or contact the sales department at sales@trolltech.com.
##
## In addition, as a special exception, Trolltech, as the sole
## copyright holder for Qt Designer, grants users of the Qt/Eclipse
## Integration plug-in the right for the Qt/Eclipse Integration to
## link to functionality provided by Qt Designer and its related
## libraries.
##
## This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
## INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
## A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
## granted herein.
##
## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
##
#############################################################################

__author__ = "tsunanet at gmail-dot-com (Benoit Sigoure)"


from PyQt4 import QtCore
from PyQt4 import QtGui

import images


class Chip(QtGui.QGraphicsItem):

  def __init__(self, color, x, y):
    QtGui.QGraphicsItem.__init__(self)

    self.x = x
    self.y = y
    self.color = color
    self.setZValue((x + y) % 2)
    self.stuff = []

    self.setFlags(QtGui.QGraphicsItem.ItemIsSelectable
                  | QtGui.QGraphicsItem.ItemIsMovable)
    self.setAcceptsHoverEvents(True)

  def boundingRect(self):
    return QtCore.QRectF(0, 0, 110, 70)

  def shape(self):
    path = QtGui.QPainterPath()
    path.addRect(14, 14, 82, 42)
    return path

  def paint(self, painter, option, unused_widget):
    if option.state & QtGui.QStyle.State_Selected:
      fillColor = self.color.dark(150)
    else:
      fillColor = self.color

    if option.state & QtGui.QStyle.State_MouseOver:
      fillColor = fillColor.light(125)

    if option.levelOfDetail < 0.2:
      if option.levelOfDetail < 0.125:
        painter.fillRect(QtCore.QRectF(0, 0, 110, 70), fillColor)
        return

      painter.setPen(QtGui.QPen(QtCore.Qt.black, 0))
      painter.setBrush(fillColor)
      painter.drawRect(13, 13, 97, 57)
      return

    oldPen = painter.pen()
    pen = oldPen
    width = 0
    if option.state & QtGui.QStyle.State_Selected:
      width += 2

    pen.setWidth(width)
    if option.state & QtGui.QStyle.State_Sunken:
      level = 120
    else:
      level = 100
    painter.setBrush(QtGui.QBrush(fillColor.dark(level)))

    painter.drawRect(QtCore.QRect(14, 14, 79, 39))
    if option.levelOfDetail >= 1:
      painter.setPen(QtGui.QPen(QtCore.Qt.gray, 1))
      painter.drawLine(15, 54, 94, 54)
      painter.drawLine(94, 53, 94, 15)
      painter.setPen(QtGui.QPen(QtCore.Qt.black, 0))

    # Draw text
    if option.levelOfDetail >= 2:
      font = QtGui.QFont("Times", 10)
      font.setStyleStrategy(QtGui.QFont.ForceOutline)
      painter.setFont(font)
      painter.save()
      painter.scale(0.1, 0.1)
      painter.drawText(170, 180,
                       "Model: VSC-2000 (Very Small Chip) at %ix%i"
                       % (self.x, self.y))
      painter.drawText(170, 200, "Serial number: DLWR-WEER-123L-ZZ33-SDSJ")
      painter.drawText(170, 220, "Manufacturer: Google Inc.")
      painter.restore()

    lines = []
    if option.levelOfDetail >= 0.5:
      if option.levelOfDetail > 0.5:
        step = 1
      else:
        step = 2
      for i in xrange(0, 11, step):
        lines.append(QtCore.QLineF(18 + 7 * i, 13, 18 + 7 * i, 5))
        lines.append(QtCore.QLineF(18 + 7 * i, 54, 18 + 7 * i, 62))
      for i in xrange(0, 7, step):
        lines.append(QtCore.QLineF(5, 18 + i * 5, 13, 18 + i * 5))
        lines.append(QtCore.QLineF(94, 18 + i * 5, 102, 18 + i * 5))
    if option.levelOfDetail >= 0.4:
      lines += [
          QtCore.QLineF(25, 35, 35, 35),
          QtCore.QLineF(35, 30, 35, 40),
          QtCore.QLineF(35, 30, 45, 35),
          QtCore.QLineF(35, 40, 45, 35),
          QtCore.QLineF(45, 30, 45, 40),
          QtCore.QLineF(45, 35, 55, 35),
          ]
    painter.drawLines(lines)

    # Draw red ink
    if len(self.stuff) > 1:
      painter.setPen(QtGui.QPen(QtCore.Qt.red, 1, QtCore.Qt.SolidLine,
                                QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
      painter.setBrush(QtCore.Qt.NoBrush)
      path = QtGui.QPainterPath()
      path.moveTo(self.stuff[0])
      for point in self.stuff:
        path.lineTo(point)
      painter.drawPath(path)

  def mousePressEvent(self, event):
    QtGui.QGraphicsItem.mousePressEvent(self, event)
    self.update()

  def mouseMoveEvent(self, event):
    if event.modifiers() & QtCore.Qt.ShiftModifier:
      self.stuff.append(event.pos())
      self.update()
      return
    QtGui.QGraphicsItem.mouseMoveEvent(self, event)

  def mouseReleaseEvent(self, event):
    QtGui.QGraphicsItem.mouseReleaseEvent(self, event)
    self.update()


class View(QtGui.QFrame):

  def __init__(self, name, parent=None):
    QtGui.QFrame.__init__(self, parent)

    self.setFrameStyle(QtGui.QFrame.Sunken | QtGui.QFrame.StyledPanel)

    self.graphicsView = QtGui.QGraphicsView()
    self.graphicsView.setRenderHint(QtGui.QPainter.Antialiasing, False)
    self.graphicsView.setDragMode(QtGui.QGraphicsView.RubberBandDrag)
    self.graphicsView.setViewportUpdateMode(
        QtGui.QGraphicsView.SmartViewportUpdate)

    size = self.style().pixelMetric(QtGui.QStyle.PM_ToolBarIconSize)
    iconSize = QtCore.QSize(size, size)

    zoomInIcon = QtGui.QToolButton()
    zoomInIcon.setAutoRepeat(True)
    zoomInIcon.setAutoRepeatInterval(33)
    zoomInIcon.setAutoRepeatDelay(0)
    zoomInIcon.setIcon(QtGui.QIcon(
      QtGui.QPixmap(":/zoomin.png")))
    zoomInIcon.setIconSize(iconSize)
    zoomOutIcon = QtGui.QToolButton()
    zoomOutIcon.setAutoRepeat(True)
    zoomOutIcon.setAutoRepeatInterval(33)
    zoomOutIcon.setAutoRepeatDelay(0)
    zoomOutIcon.setIcon(QtGui.QIcon(
      QtGui.QPixmap(":/zoomout.png")))
    zoomOutIcon.setIconSize(iconSize)
    self.zoomSlider = QtGui.QSlider()
    self.zoomSlider.setMinimum(0)
    self.zoomSlider.setMaximum(500)
    self.zoomSlider.setValue(250)
    self.zoomSlider.setTickPosition(QtGui.QSlider.TicksRight)

    # Zoom slider layout
    zoomSliderLayout = QtGui.QVBoxLayout()
    zoomSliderLayout.addWidget(zoomInIcon)
    zoomSliderLayout.addWidget(self.zoomSlider)
    zoomSliderLayout.addWidget(zoomOutIcon)

    rotateLeftIcon = QtGui.QToolButton()
    rotateLeftIcon.setIcon(QtGui.QIcon(
      QtGui.QPixmap(":/rotateleft.png")))
    rotateLeftIcon.setIconSize(iconSize)
    rotateRightIcon = QtGui.QToolButton()
    rotateRightIcon.setIcon(QtGui.QIcon(
      QtGui.QPixmap(":/rotateright.png")))
    rotateRightIcon.setIconSize(iconSize)
    self.rotateSlider = QtGui.QSlider()
    self.rotateSlider.setOrientation(QtCore.Qt.Horizontal)
    self.rotateSlider.setMinimum(-360)
    self.rotateSlider.setMaximum(360)
    self.rotateSlider.setValue(0)
    self.rotateSlider.setTickPosition(QtGui.QSlider.TicksBelow)

    # Rotate slider layout
    rotateSliderLayout = QtGui.QHBoxLayout()
    rotateSliderLayout.addWidget(rotateLeftIcon)
    rotateSliderLayout.addWidget(self.rotateSlider)
    rotateSliderLayout.addWidget(rotateRightIcon)

    self.resetButton = QtGui.QToolButton()
    self.resetButton.setText("0")
    self.resetButton.setEnabled(False)

    # Label layout
    labelLayout = QtGui.QHBoxLayout()
    self.label = QtGui.QLabel(name)
    self.antialiasButton = QtGui.QToolButton()
    self.antialiasButton.setText("Antialiasing")
    self.antialiasButton.setCheckable(True)
    self.antialiasButton.setChecked(False)

    # No openGlButton
    # No printButton

    labelLayout.addWidget(self.label)
    labelLayout.addStretch()
    labelLayout.addWidget(self.antialiasButton)

    topLayout = QtGui.QGridLayout()
    topLayout.addLayout(labelLayout, 0, 0)
    topLayout.addWidget(self.graphicsView, 1, 0)
    topLayout.addLayout(zoomSliderLayout, 1, 1)
    topLayout.addLayout(rotateSliderLayout, 2, 0)
    topLayout.addWidget(self.resetButton, 2, 1)
    self.setLayout(topLayout)

    self.connect(self.resetButton, QtCore.SIGNAL("clicked()"),
                 self.resetView)
    self.connect(self.zoomSlider, QtCore.SIGNAL("valueChanged(int)"),
                 self.setupMatrix)
    self.connect(self.rotateSlider, QtCore.SIGNAL("valueChanged(int)"),
                 self.setupMatrix)
    self.connect(self.graphicsView.verticalScrollBar(),
                 QtCore.SIGNAL("valueChanged(int)"),
                 self.setResetButtonEnabled)
    self.connect(self.graphicsView.horizontalScrollBar(),
                 QtCore.SIGNAL("valueChanged(int)"),
                 self.setResetButtonEnabled)
    self.connect(self.antialiasButton, QtCore.SIGNAL("toggled(bool)"),
                 self.toggleAntialiasing)
    self.connect(rotateLeftIcon, QtCore.SIGNAL("clicked()"),
                 self.rotateLeft)
    self.connect(rotateRightIcon, QtCore.SIGNAL("clicked()"),
                 self.rotateRight)
    self.connect(zoomInIcon, QtCore.SIGNAL("clicked()"),
                 self.zoomIn)
    self.connect(zoomOutIcon, QtCore.SIGNAL("clicked()"),
                 self.zoomOut)

    self.setupMatrix()

  def view(self):
    return self.graphicsView

  def resetView(self):
    self.zoomSlider.setValue(250)
    self.rotateSlider.setValue(0)
    self.setupMatrix()
    self.graphicsView.ensureVisible(QtCore.QRectF(0, 0, 0, 0))

    self.resetButton.setEnabled(False)

  def setResetButtonEnabled(self):
    self.resetButton.setEnabled(True)

  def setupMatrix(self):
    scale = pow(2.0, (self.zoomSlider.value() - 250) / 50.0)

    matrix = QtGui.QMatrix()
    matrix.scale(scale, scale)
    matrix.rotate(self.rotateSlider.value())

    self.graphicsView.setMatrix(matrix)
    self.setResetButtonEnabled()

  def toggleAntialiasing(self):
    self.graphicsView.setRenderHint(QtGui.QPainter.Antialiasing,
                                    self.antialiasButton.isChecked())

  def zoomIn(self):
    self.zoomSlider.setValue(self.zoomSlider.value() + 1)

  def zoomOut(self):
    self.zoomSlider.setValue(self.zoomSlider.value() - 1)

  def rotateLeft(self):
    self.rotateSlider.setValue(self.rotateSlider.value() - 10)

  def rotateRight(self):
    self.rotateSlider.setValue(self.rotateSlider.value() + 10)


class MainWindow(QtGui.QWidget):

  def __init__(self, parent=None):
    QtGui.QWidget.__init__(self, parent)

    self.populateScene()

    self.h1Splitter = QtGui.QSplitter()
    self.h2Splitter = QtGui.QSplitter()

    vSplitter = QtGui.QSplitter()
    vSplitter.setOrientation(QtCore.Qt.Vertical)
    vSplitter.addWidget(self.h1Splitter)
    vSplitter.addWidget(self.h2Splitter)

    view = View("Top left view")
    view.view().setScene(self.scene)
    self.h1Splitter.addWidget(view)

    view = View("Top right view")
    view.view().setScene(self.scene)
    self.h1Splitter.addWidget(view)

    view = View("Bottom left view")
    view.view().setScene(self.scene)
    self.h2Splitter.addWidget(view)

    view = View("Bottom right view")
    view.view().setScene(self.scene)
    self.h2Splitter.addWidget(view)

    layout = QtGui.QHBoxLayout()
    layout.addWidget(vSplitter)
    self.setLayout(layout)

    self.setWindowTitle("Chip Demo")

  def populateScene(self):
    self.scene = QtGui.QGraphicsScene()

    image = QtGui.QImage(":/qt4logo.png")

    # Populate scene
    xx = 0
    nitems = 0
    for i in xrange(-11000, 11000, 110):
      xx += 1
      yy = 0
      for j in xrange(-7000, 7000, 70):
        yy += 1
        x = (i + 11000) / 22000.0
        y = (j + 7000) / 14000.0

        color = QtGui.QColor(image.pixel(int(image.width() * x),
                                         int(image.height() * y)))
        item = Chip(color, xx, yy)
        item.setPos(QtCore.QPointF(i, j))
        self.scene.addItem(item)

        nitems += 1

def main(argv):
  app = QtGui.QApplication(argv)

  window = MainWindow()
  window.show()

  return app.exec_()


if __name__ == "__main__":
  import sys
  sys.exit(main(sys.argv))
